##########################################################################################################################
# Main makefile for unit tests
##########################################################################################################################

######################################
# target
######################################
TARGET = mississippi-mcu-packetusart-testbed
COMPONENT_DEPS =

######################################
# building variables
######################################
# debug build?
DEBUG = 1
# save preprocessor output
SAVETEMPS = 0
# optimization
OPT = -Og
# c++ standard
CXXSTD = -std=c++0x


#######################################
# paths
#######################################
# Build path
BUILD_DIR = build_tests
COMPONENTS = components
PLATFORMS = platforms
PLATFORM = platform-test-f4-ll
CONFIG_OVERRIDES = config-overrides

######################################
# source
######################################
# C sources
C_SOURCES = \

CXX_SOURCES = \

# ASM sources
ASM_SOURCES =  \

#######################################
# binaries
#######################################
PREFIX =
# The gcc compiler bin path can be either defined in make command via GCC_PATH variable (> make GCC_PATH=xxx)
# either it can be added to the PATH environment variable.
ifdef GCC_PATH
CC = $(GCC_PATH)/$(PREFIX)gcc
CXXC = $(GCC_PATH)/$(PREFIX)g++
LD =  $(GCC_PATH)/$(PREFIX)g++
AS = $(GCC_PATH)/$(PREFIX)gcc -x assembler-with-cpp
CP = $(GCC_PATH)/$(PREFIX)objcopy
SZ = $(GCC_PATH)/$(PREFIX)size
else
CC = $(PREFIX)gcc
CXXC = $(PREFIX)g++
AS = $(PREFIX)gcc -x assembler-with-cpp
LD =  $(PREFIX)g++
CP = $(PREFIX)objcopy
SZ = $(PREFIX)size
endif
HEX = $(CP) -O ihex
BIN = $(CP) -O binary -S

#######################################
# CFLAGS
#######################################
# cpu
CPU = -m32

# macros for gcc
# AS defines
AS_DEFS = 

# C defines
COMMON_DEFS = \
-DUNITTEST \

C_DEFS =  \

CXX_DEFS = \

# AS includes
AS_INCLUDES = 

# C includes
COMMON_INCLUDES =  \
-I$(COMPONENTS) \

C_INCLUDES =  \

CXX_INCLUDES =  \

LIBS =  
LIBDIR = 

# compile gcc flags
ASFLAGS = $(CPU) $(AS_DEFS) $(AS_INCLUDES) $(OPT) -Wall

CFLAGS = $(CPU) $(COMMON_DEFS) $(C_DEFS) $(COMMON_INCLUDES) $(C_INCLUDES) $(OPT) -Wall
CXXFLAGS = $(CPU) $(COMMON_DEFS) $(CXX_DEFS) $(COMMON_INCLUDES) $(CXX_INCLUDES) $(OPT) $(CXXSTD) -Wall

# default action: build all
.PONY: all clean

all: $(BUILD_DIR)/$(TARGET)


#######################################
# components
#######################################
include $(sort $(wildcard $(COMPONENTS)/*/test.mk))
include $(PLATFORMS)/$(PLATFORM)/component.mk

ifeq ($(DEBUG), 1)
CFLAGS += -g3
CXXFLAGS += -g3
endif

ifeq ($(SAVETEMPS), 1)
CFLAGS += -save-temps
CXXFLAGS += -save-temps
endif

# Generate dependency information
CFLAGS += -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@)"
CXXFLAGS += -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@)"


#######################################
# LDFLAGS
#######################################
# libraries
LDFLAGS = $(CPU) -Wl,-Map=$(BUILD_DIR)/$(TARGET).map,--cref $(LIBDIR) $(LIBS)

ifeq ($(MKDBG), 1)
 $(info >>--------------------------------------------)
 $(info $$C_SOURCES is [${C_SOURCES}])
 $(info )
 $(info $$CXX_SOURCES is [${CXX_SOURCES}])
 $(info )
 $(info $$COMMON_DEFS is [${COMMON_DEFS}])
 $(info )
 $(info $$C_DEFS is [${C_DEFS}])
 $(info )
 $(info $$CXX_DEFS is [${CXX_DEFS}])
 $(info )
 $(info $$COMMON_INCLUDES is [${COMMON_INCLUDES}])
 $(info )
 $(info $$C_INCLUDES is [${C_INCLUDES}])
 $(info )
 $(info $$CXX_INCLUDES is [${CXX_INCLUDES}])
 $(info )
 $(info $$CFLAGS is [${CFLAGS}])
 $(info )
 $(info $$CXXFLAGS is [${CXXFLAGS}])
 $(info )
 $(info $$LIBDIR is [${LIBDIR}])
 $(info )
 $(info $$LIBS is [${LIBS}])
 $(info )
 $(info $$COMPONENT_DEPS is [${COMPONENT_DEPS}])
 $(info <<--------------------------------------------)
endif

#######################################
# build the application
#######################################
# list of objects
OBJECTS = $(addprefix $(BUILD_DIR)/,$(C_SOURCES:.c=.o))
vpath %.c $(sort $(dir $(C_SOURCES)))

# list of ASM program objects
OBJECTS +=$(addprefix $(BUILD_DIR)/,$(ASM_SOURCES:.s=.o))
vpath %.s $(sort $(dir $(ASM_SOURCES)))

OBJECTS +=$(addprefix $(BUILD_DIR)/,$(CXX_SOURCES:.cpp=.o))
vpath %.cpp $(sort $(dir $(CXX_SOURCES)))

$(BUILD_DIR)/%.o: %.c Makefile firmware.mk $(COMPONENT_DEPS) | $(BUILD_DIR)
	mkdir -p $(@D)
	$(CC) -c $(CFLAGS) -Wa,-a,-ad,-alms=$(BUILD_DIR)/$(<:.c=.lst) $< -o $@

$(BUILD_DIR)/%.o: %.cpp Makefile firmware.mk $(COMPONENT_DEPS) | $(BUILD_DIR) 
	mkdir -p $(@D)
	$(CXXC) -c $(CXXFLAGS) -Wa,-a,-ad,-alms=$(BUILD_DIR)/$(<:.cpp=.lst) $< -o $@

$(BUILD_DIR)/%.o: %.s Makefile firmware.mk $(COMPONENT_DEPS) | $(BUILD_DIR)
	mkdir -p $(@D)
	$(AS) -c $(CFLAGS) $< -o $@

$(BUILD_DIR)/$(TARGET): $(OBJECTS) Makefile firmware.mk
	$(LD) $(OBJECTS) $(LDFLAGS) -o $@
	$(SZ) $@

$(BUILD_DIR):
	mkdir -p $@

#######################################
# clean up
#######################################
clean:
	-rm -fR $(BUILD_DIR)
  
#######################################
# dependencies
#######################################
-include $(wildcard $(BUILD_DIR)/*.d)

# *** EOF ***
